TOOL.Category		= "Construction"
TOOL.Name			= "#Passenger"
TOOL.Command		= nil
TOOL.ConfigName		= ""

TOOL.ClientConVar["model"]	= "models/Nova/airboat_seat.mdl";
TOOL.ClientConVar["offset_x"] = 0;
TOOL.ClientConVar["offset_y"] = 0;
TOOL.ClientConVar["offset_z"] = 0;
TOOL.ClientConVar["rot_x"] = 0;
TOOL.ClientConVar["rot_y"] = 0;
TOOL.ClientConVar["rot_z"] = 0;
TOOL.ClientConVar["color_r"] = 255;
TOOL.ClientConVar["color_g"] = 255;
TOOL.ClientConVar["color_b"] = 255;
TOOL.ClientConVar["color_a"] = 255;

list.Set( "SeatModels", "models/Nova/airboat_seat.mdl", {} );
list.Set( "SeatModels", "models/Nova/chair_office01.mdl", {} );
list.Set( "SeatModels", "models/Nova/chair_office02.mdl", {} );
list.Set( "SeatModels", "models/Nova/chair_plastic01.mdl", {} );
list.Set( "SeatModels", "models/Nova/jalopy_seat.mdl", {} );
list.Set( "SeatModels", "models/Nova/jeep_seat.mdl", {} );

if (CLIENT) then
	language.Add("Tool_passenger_name", "Passenger Seat")
	language.Add("Tool_passenger_desc", "Apply a passenger seat to a vehicle." )
	language.Add("Tool_passenger_0", "Click to create a passenger seat." )
	language.Add("Undone_passenger", "Passenger Seat")
end;

function TOOL:LeftClick(trace)
	local player = self:GetOwner();
	local model = self:GetClientInfo("model");

	if ( !util.IsValidModel(model) ) then
		return false;
	end;

	if (!trace.Entity or !trace.Entity:IsValid() or trace.Entity:GetClass() != "prop_vehicle_jeep" and trace.Entity:GetClass() != "prop_vehicle_airboat") then
		return false;
	end;

	local vehicle = trace.Entity;
	local offset = {
		x = self:GetClientNumber("offset_x", 0),
		y = self:GetClientNumber("offset_y", 0),
		z = self:GetClientNumber("offset_z", 0)
	};
	local rot = {
		x = self:GetClientNumber("rot_x", 0),
		y = self:GetClientNumber("rot_y", 0),
		z = self:GetClientNumber("rot_z", 0),
	};
	local color = {
		r = self:GetClientNumber("color_r", 255),
		g = self:GetClientNumber("color_g", 255),
		b = self:GetClientNumber("color_b", 255),
		a = self:GetClientNumber("color_a", 255),
	};

	local seatEntity = ents.Create("prop_vehicle_prisoner_pod");
	
	if ( seatEntity and seatEntity:IsValid() ) then
		seatEntity:SetModel(model or "models/nova/jeep_seat.mdl");
		seatEntity:SetKeyValue("vehiclescript", "scripts/vehicles/prisoner_pod.txt");
		seatEntity:SetAngles( trace.Entity:GetAngles() + Vector(rot.x, rot.y, rot.z) );
		seatEntity:SetPos( vehicle:GetPos() + Vector(offset.x, offset.y, offset.z) );
		seatEntity:SetColor(color.r, color.g, color.b, color.a);
		seatEntity:Spawn();
		seatEntity:SetSolid(SOLID_VPHYSICS);
		seatEntity:SetMoveType(MOVETYPE_NONE);
		seatEntity:SetParent(trace.Entity);
		seatEntity:Activate();
		DoPropSpawnedEffect(seatEntity);

		undo.Create("passenger seat");
			undo.AddEntity(seatEntity);
			undo.SetPlayer(player);
		undo.Finish("passenger seat");

		return true;
	end;

	return false;
end

function TOOL:RightClick( trace )
	return self:LeftClick(trace);
end;

local function tool_ResetDefaultValues(player)
	player:ConCommand("passenger_offset_x 0\n");
	player:ConCommand("passenger_offset_y 0\n");
	player:ConCommand("passenger_offset_z 0\n");
	player:ConCommand("passenger_rot_x 0\n");
	player:ConCommand("passenger_rot_y 0\n");
	player:ConCommand("passenger_rot_z 0\n");
end;

concommand.Add("passenger_reset", function(player, command, arguments)
	tool_ResetDefaultValues(player);
end);

function TOOL.BuildCPanel(controlPanel)
	controlPanel:AddControl( "Header", {Text = "#Tool_passenger_name", Description = "#Tool_passenger_desc"} );
	controlPanel:AddControl( "PropSelect", {Label = "Seat Model:", ConVar = "passenger_model", Models = list.Get("SeatModels")} );
	controlPanel:AddControl( "Slider", {Label = "X-Axis Offset", Type = "Float", Min = "-128", Max = "128", Command = "passenger_offset_x"} );
	controlPanel:AddControl( "Slider", {Label = "Y-Axis Offset", Type = "Float", Min = "-128", Max = "128", Command = "passenger_offset_y"} );
	controlPanel:AddControl( "Slider", {Label = "Z-Axis Offset", Type = "Float", Min = "-128", Max = "128", Command = "passenger_offset_z"} );
	controlPanel:AddControl( "Slider", {Label = "X-Axis Rotation", Type = "Float", Min = "0", Max = "360", Command = "passenger_rot_x"} );
	controlPanel:AddControl( "Slider", {Label = "Y-Axis Rotation", Type = "Float", Min = "0", Max = "360", Command = "passenger_rot_y"} );
	controlPanel:AddControl( "Slider", {Label = "Z-Axis Rotation", Type = "Float", Min = "0", Max = "360", Command = "passenger_rot_z"} );
	controlPanel:AddControl( "Button", {Label = "Reset the options for rotation and offset.", Command = "passenger_reset", Text = "Reset Options"} );
	controlPanel:AddControl( "Color", {Label = "Seat Color", Red = "passenger_color_r", Blue = "passenger_color_b", Green = "passenger_color_g", Alpha = "passenger_color_a", ShowHSV = 1, ShowRGB = 1, Multiplier = 255} );
end;

function TOOL:UpdateGhostModel(entity, player, position, angle)
	if ( !entity or !entity:IsValid() ) then return; end;

	local tr = utilx.GetPlayerTrace( player, player:GetCursorAimVector() );
	local trace = util.TraceLine( tr );
	if (!trace.Hit) then return; end;
	
	if ( trace.Entity:IsPlayer() ) then
		entity:SetNoDraw(true);
		return;
	end;
	
	entity:SetPos(position);
	entity:SetAngles(angle);
	entity:SetNoDraw(false);
end;

function TOOL:Think()
	local player = self:GetOwner();
	local entity = player:GetEyeTraceNoCursor().Entity;
	local model = self:GetClientInfo("model");
	local offset = {
		x = self:GetClientNumber("offset_x", 0),
		y = self:GetClientNumber("offset_y", 0),
		z = self:GetClientNumber("offset_z", 0)
	};
	local rot = {
		x = self:GetClientNumber("rot_x", 0),
		y = self:GetClientNumber("rot_y", 0),
		z = self:GetClientNumber("rot_z", 0),
	};
	local position = entity:GetPos() + Vector(offset.x, offset.y, offset.z);
	local angle = entity:GetAngles() + Angle(rot.x, rot.y, rot.z);

	if (entity:GetClass() == "prop_vehicle_jeep" or entity:GetClass() == "prop_vehicle_airboat") then
		if (!self.GhostEntity or !self.GhostEntity:IsValid() or self.GhostEntity:GetModel() != model) then
			self:MakeGhostEntity(model, position, angle);
		end;

		self:UpdateGhostModel(self.GhostEntity, player, position, angle);
	end;
end;